#include "asm86.h"         
#include "ti86asm.inc"
#include "ti86ops.inc"     ;stuff to deal with OP registers
#include "ti86abs.inc"     ;absolute addressing

.org _asm_exec_ram
    nop
    jp Begin                  ;can't be jr... arrrgh!
    .dw 0
    .dw Title
Title:
.db "Monolyth Sprite Editor",0


_div_hl_a  equ $4048


Begin:
    call _flushallmenus      ;no menus up
    res appAutoScroll,(iy+appflags) ;don't scroll bottom line
    res indicRun,(iy+indicflags)  ; runindicoff
    res textwrite, (iy+new_grf_flgs) 
    call DeZCP                       ;display the title screen
    xor a
    ld b,255
    ld hl, _plotSScreen
    call FillMem                  ; clear data storage (plot screen)


LoadFile:
     call GetLoadName
     call Load
     cp $FF                       ;error?  
     jp z, EndProgram             ; then quit
                                  ; all variables are set up now
     jr UpdateDsp 

NextSprite:                       ;why the hell is this here? so we can use jr instead of jp
 ld hl,(FileOffset)
 ld de,8
 add hl,de
 ex de,hl
 ld hl,(FileSize)
 call CP_HL_DE         ;if the offset+8 = the size, no more sprites to go to
 jr z, UpdateDsp
 ex de,hl
 ld (FileOffset),hl    ;no, then add 8 bytes to it.

 ld hl,(SpritePtr)     ;and 8 to the sprite pointer too
 ld de,8
 add hl,de
 ld (SpritePtr), hl     
 jr UpdateDsp

NextSp16:                       
 ld hl,(FileOffset)
 ld de,128
 add hl,de
 ex de,hl
 ld hl,(FileSize)
 dec hl
 call CP_HL_DE         ;if the offset+128 >= the size-1, no more sprites to go to
 jr c, UpdateDsp
 ex de,hl
 ld (FileOffset),hl    ;no, then add 128 bytes to it.

 ld hl,(SpritePtr)     ;and 128 to the sprite pointer too
 ld de,128
 add hl,de
 ld (SpritePtr), hl     
 jr UpdateDsp

PrevSprite:
 ld hl,(FileOffset)
 ld de,0
 call CP_HL_DE         ;if the offset=0, no more sprites to go to
 jr z, UpdateDsp
 ld de,8
 sbc hl,de
 ld (FileOffset),hl    ;no, then add subtract 8 bytes.

 ld hl,(SpritePtr)     ;and from the sprite pointer too
 ld de,8
 sbc hl,de
 ld (SpritePtr), hl     
 jr UpdateDsp
 


UpdateDsp:
 call DispWorkSpace
KeyLoop:
 call _getkey
 cp kRight
 jr z, Right
 cp kLeft
 jr z, Left
 cp kDown
 jr z, Down
 cp kUp
 jr z, Up
 cp kF1
 jr z, FlipPixel
 cp kAdd
 jr z, NextSprite
 cp kSub
 jr z, PrevSprite
 cp kF4 
 jp z, PrevSp16
 cp kF5
 jr z, NextSp16
 cp kExit
 jr z,EndProgram
 jr KeyLoop

EndProgram:
    set appAutoScroll,(iy+appflags)
    call _clrScrn       ;and clear the screen one last time
    call _homeup        ; -> 0,0
    xor a
    ld b,255
    ld hl, _plotSScreen
    call FillMem                  ; clear data storage (plot screen)
    ret

Right:
 ld a,(XPos)
 cp 7
 jr z, UpdateDsp
 inc a
 ld (XPos),a
 jr UpdateDsp
Left:
 ld a,(XPos)
 or a                ;cp 0
 jr z, UpdateDsp
 dec a
 ld (XPos),a
 jr UpdateDsp
Down:
 ld a,(YPos)
 cp 7                ;cp 0
 jr z, UpdateDsp
 inc a
 ld (YPos),a
 jr UpdateDsp
Up:
 ld a,(YPos)
 or a                ;cp 0
 jr z, UpdateDsp
 dec a
 ld (YPos),a
 jr UpdateDsp

FlipPixel:
 ld hl, (SpritePtr)                 ;go to the sprite pointer
 ld a, (YPos)
 ld b,0
 ld c,a
 add hl,bc                          ;add hl,a (hl points to right byte)
 ld a, (XPos)
 ld b, a
 ld c,%10000000
FindX:
 rrc c                              ;move to the right 1 (until at proper pixel)
 djnz FindX                         
 ld a,(hl)                          ;a is the sprite value
 xor c
 ld (hl),a 
 jp UpdateDsp


;hl points to sprite, a to offset on screen

DispWorkSpace:
 ld hl, (SpritePtr)                 ;go to the sprite pointer
 ld bc, $37F                    ;offset (in this case, the bottom right)
 call PutSprite
 ld bc,0
 ld (_curRow),bc     ;cursor ->0,0
DispLoop:
 ld a,(hl)           ;ld first line of sprite
Bit7:
 ld b,$DF             ;b=space char
 bit 7,a             ;test bit 7
 call z, SetWhite
 call DispPixel
Bit6:
 ld b,$DF             ;b=space char
 bit 6,a             ;test bit 7
 call z, SetWhite
 call DispPixel
Bit5:
 ld b,$DF             ;b=space char
 bit 5,a             ;test bit 7
 call z, SetWhite
 call DispPixel
Bit4:
 ld b,$DF             ;b=space char
 bit 4,a             ;test bit 7
 call z, SetWhite
 call DispPixel
Bit3:
 ld b,$DF             ;b=space char
 bit 3,a             ;test bit 7
 call z, SetWhite
 call DispPixel
Bit2:
 ld b,$DF            ;b=space char
 bit 2,a             ;test bit 7
 call z, SetWhite
 call DispPixel
Bit1:
 ld b,$DF             ;b=space char
 bit 1,a             ;test bit 7
 call z, SetWhite
 call DispPixel
Bit0:
 ld b,$DF             ;b=space char
 bit 0,a             ;test bit 7
 call z, SetWhite
 call DispPixel
NewLine: 
 ld a,(_curRow)
 inc a
 cp 8               ;last line? go to the next update part
 jr z,NumStats
 ld (_curRow),a     ;increase cursor row (y)
 ld a,(_curCol)
 xor a          
 ld (_curCol),a    ;column -> 0 (x)
 inc hl            ;add 1 to sprite pointer
 jr DispLoop       ;go back + do it again
 
SetWhite:
  ld b,$20              ;sets b to blank
  ret
DispPixel:
  push af               ;displays b as an ascii char, inverts it if it's at curpos
  push hl
  push bc
  ld a,(_curRow)
  ld hl,YPos
  ld b,(hl)           ;y column of pos,curpos matched?
  cp b
  call z, TestX
  pop bc              
  ld a,b
  call _putc
  pop hl
  pop af
  res textInverse,(iy+textflags)  ;no longer inverse
  ret

TestX:
  ld a,(_curCol)
  ld hl,XPos
  ld b,(hl)           ;x row matched?
  cp b
  jr z, SetIt
  ret                   ;no, so just leave
SetIt:                  ;yes, so set invert flag
  set textInverse,(iy+textflags)
  ret  


NumStats:              ;displays the sprite #, etc.
  ld hl,$0C01 
  ld (_curRow),hl
  ld hl,MaxText
  call _puts
  ld hl,(FileSize)
  ld a,8
  call _div_hl_a
  dec hl               ;Sprite number :-)
  call DispHL
  ld hl,$0C00 
  ld (_curRow),hl
  ld hl,SpriteText
  call _puts
  ld hl,(FileOffset)
  ld a,8
  call _div_hl_a
  call DispHL
  ret

  
; hl = ptr to sprite, bc = screen offset

PutSprite:
  push hl
  ex de,hl         ;de now points to sprite
  ld hl,$FC00      ;hl to video mem
  add hl,bc
  ld b,8
BlitLoop:
  ld a,(de)
  ld (hl),a       ;blit a line
  inc de           ;move to next line
  push de          
  ld de,16
  add hl,de          ;add to address
  pop de
  djnz BlitLoop      ;repeat until done
  pop hl
  ret

PrevSp16:                       
 ld hl,(FileOffset)
 ld de,128
 xor a               ;clear carry flag
 sbc hl,de
 jp m, UpdateDsp     ;if it's zero or minus, exit
 jp z, UpdateDsp
 ld (FileOffset),hl    
 ld hl,(SpritePtr)     
 xor a                ;clear carry flag
 ld de,128
 sbc hl,de
 ld (SpritePtr), hl     
 jp UpdateDsp



GetLoadName:
  call LoadScreen
LoadLoop:
    halt                    ;waste time / save battery
    call _getky             ;gets the key

    cp K_ALPHA
    call z, ToggleAlpha
    cp K_SECOND
    call z, ToggleNum
    cp K_ENTER
    jr z, EnterExit
    cp K_LEFT
    jr z, BkSpc
    cp K_DEL
    jr z, BkSpc

    call ToAscii            ;convert the code to ascii
    or a                    ;cp 0 - still nothing?
    jr z,LoadLoop               

    push af
    ld a,(FileName)
    cp 8
    jr z, PopLoop
    inc a                   ; length=length+1
    ld (FileName),a         ; store length
    ld hl,FileName          
    ld c,a
    ld b,0                 ;bc=a
    add hl,bc                ;hl + length = offset for character
    pop af
    ld (hl),a
    ld hl,FileName
    ld bc,$0701
    ld (_curRow),bc
    call _putps              ;display onscreen
    jr LoadLoop

PopLoop:
  pop af
  jr LoadLoop

BkSpc:
  ld a,(FileName)
  or a
  jr z, LoadLoop          ; can't backspace if no string
  dec a                   ; length=length-1
  ld (FileName),a         ; store length
  or a                    ; 
  jr z, GetLoadName       ; if 0, can't display; redraw screen 
  call LoadScreen         ;redraw screen
  ld hl,FileName
  ld bc,$0701
  ld (_curRow),bc
  call _putps             ;display string onscreen
  jr LoadLoop

EnterExit:
  ld a,(FileName)
  or a                      ;cp 0
  jr z, LoadLoop                     ; if length still 0, can't enter..
  ld hl,$0000              ;cursor to upper right
  ld (_curRow),hl
  call _clrScrn          
  ret

LoadScreen:
  call _clrScrn
  ld bc, $0000
  ld (_curRow),bc
  ld hl,StringInputText
  call _puts                             ;big font

  ld bc,$0601
  ld (_curRow),bc
  ld a,'>'
  call _putc
  ld bc,$0F01
  ld (_curRow),bc
  ld a,'<'
  call _putc
  ret


Load:
  ld hl,FileName-1  
  call _MOV10TOOP1       ;load filename to OP1
  call _FINDSYM          ;look it up
  jr c, NotExist
  cp $0c
  jp nz, Error
Ok:                      ;after FINDSYM: var data abs ptr in BDE 
  ld a,b
  ex de,hl               ;now in ahl
  call _inc_ptr_bde      ; incs bde to past the size word
  call _inc_ptr_bde      
  push af
  ld a,b
  ld (FileAddr_a),a
  ld (FileAddr_hl),de
  pop af
  call _load_ram_ahl     ;hl points to (ahl), which is the size word
  push hl
  call  $4010            ; ld hl,(hl) 
  ld (FileSize), hl      ;FileSize is now correct
  pop hl
  inc hl
  inc hl
  ld (SpritePtr), hl     ;points to first sprite
  ret

NotExist:
  call _clrScrn
  ld bc, $0000
  ld (_curRow),bc
  ld hl, NoExist
  call _puts
  ld bc, $0600
  ld (_curRow),bc
  ld hl,FileName
  call _putps
  ld hl,YesNo
  call _puts
YesNoLoop:
  call _getkey
  cp k9
  jr z, Error
  cp k0
  jr z, Yes
  jr YesNoLoop
Error:                    ;set when error occurs
  ld a,$FF
  ret
Yes:
  ld bc, $0000
  ld (_curRow),bc
  call _clrScrn
  ld hl,SizeText
  call _puts
  ld hl, (FileSize)
  call DispHL                  ;  display file size
  ld bc, $0001
  ld (_curRow),bc
  ld hl,SpriteText
  call _puts
  ld hl,(FileSize)
  ld a,8
  call _div_hl_a
  call DispHL                  ;display number of sprites

SizeLoop:
  call _getkey
  cp kLeft
  jr z, DecrSize
  cp kRight
  jr z, IncrSize
  cp kUp
  jr z, IncrMore
  cp kDown
  jr z, DecrMore
  cp kEnter
  jr z, DoneSize
  jr SizeLoop
DecrSize:
  ld hl, (FileSize)
  ld bc,8
  sbc hl,bc
  ld (FileSize),hl
  jr Yes                       ;display new size
DecrMore:
  ld hl, (FileSize)
  ld bc,256
  sbc hl,bc
  ld (FileSize),hl
  jr Yes                       ;display new size
IncrSize:
  ld hl, (FileSize)
  ld bc,8
  add hl,bc
  ld (FileSize),hl
  jr Yes
IncrMore:
  ld hl, (FileSize)
  ld bc,256
  add hl,bc
  ld (FileSize),hl
  jr Yes

DoneSize:
  ld bc, $0000
  ld (_curRow),bc
  call _clrScrn
  ld hl,(FileSize)
  ld de,0
  call CP_HL_DE
  jp z, Yes     ;no 0-length strings!
  call _CREATESTRNG
  jp Load                     ;created it, now load it

;---------==============================================---------
;ToAscii - converts the get_key value in a to an ascii code
;---------==============================================---------
ToAscii:
  push af                            ; save af
  ld a,(AlphaStatus)                 ; a = alpha status
  ld hl,InputTableUcase
  cp 1                               ; is a ucase?
  jr z, donewithinput               ; hl is already ucase, so done.
  ld hl,InputTableNum
  cp 2                               ; hl is num - is a?
  jr z, donewithinput               ;yes, done
  ld hl,InputTableLcase              ; now hl = lcase
donewithinput: 
  pop af                             ;restore a, which is the keycode
  ld c,a
  ld b,0            
  add hl,bc                          ;hl = hl + a (look up in table)
  ld a,(hl)                          ;new a from table
  ret                               

;---------==============================================---------
;Miscellaneous small routines
;---------==============================================---------


ToggleAlpha:
  ld a,(AlphaStatus)
  or a                      ;cp 0
  jp z,alphanoreset
  xor a                              ; a=0
  ld (AlphaStatus),a
  ret
alphanoreset:
  ld a,1
  ld (AlphaStatus),a
  xor a
  ret

ToggleNum:
  ld a,(AlphaStatus)
  or a                    ; cp 0
  jp z,numnoreset
  xor a                              ; a=0
  ld (AlphaStatus),a
  ret
numnoreset:
  ld a,2
  ld (AlphaStatus),a
  xor a
  ret


Pause:
  halt
  call _getkey
  or a
  jr z, Pause
  ret 

FillMem:                 ; takes b, number of blocks, a, value, and hl,location
  ld (hl),a              ; fill 'em with a
  inc hl                 ; next byte
  djnz FillMem           ; until done
  ret                    ; return


DispHL:                         ; from conv.asm ($4a33 is too weird)
         push     bc
         ld       de,up_data+4
         ld       b,5
ldhld:   call     UNPACK_HL
         add      a,48
         ld       (de),a
         dec      de
         djnz     ldhld
         ld       hl,up_data
         ld       b,4
lis:     ld       a,(hl)
         cp       48
         jr       nz,dis
         ld       (hl),32
         inc      hl
         djnz     lis
dis:     ld       hl,up_data
         call     _puts
         pop      bc
         ret
up_data: .db      "     ",0

DeZCP:
 ld hl,TitlePic
 push   HL
 call   _clrLCD
 pop    IX
 ld     HL, $FC00
 ld     C, 128
;Now:
; A 	temporary storage
; B	loop counter
; C	'bit counter'
; DE	pointer to Data var
; HL	VIDEO_MEM
; IX	the compressed data
CLoop:
 inc    IX
 ld     A,(IX)
 and    A
 jr     Z,EndDecode
 ld	E, A
 bit    7,A
 jr     NZ,Copy
Stream:
 and    $3F
 ld     B, A
StreamLoop:
 bit	6, E
 jr	z, DrawNoP
 ld     A, (HL)
 or     C
 ld     (HL), A
DrawNoP:
 rrc    C
 jr     NC, GoOnZCP1
 inc    HL
GoOnZCP1:
 djnz   StreamLoop
 jr     CLoop
Copy:
 rl	E
 ld     B, 7
CopyLoop:
 rl	E
 jr	nc, DrawNoPoint
 ld     A, (HL)
 or     C
 ld     (HL), A
DrawNoPoint:
 rrc    C
 jr     nc, GoOnZCP
 inc    HL
GoOnZCP:
 djnz   CopyLoop
 jr     CLoop
EndDecode:
 call _getkey
 call _clrLCD
 ret



AlphaStatus = _plotSScreen
FileName = _plotSScreen+1
FileSize = _plotSScreen+11
FileAddr_a = _plotSScreen+13
FileAddr_hl = _plotSScreen+14
XPos = _plotSScreen+16
YPos = _plotSScreen+17
SpritePtr = _plotSScreen+18
FileOffset = _plotSScreen+20

StringInputText:
 .db "String Name?",0

SpriteText:
 .db "Sp#:",0
MaxText:
 .db "Max:",0

NoExist:
 .db "Make ",$22,0
YesNo:
 .db $22,"? (Y/N)",0

SizeText:
 .db "Size (",$06,$07,$05,$CF,"):",0

InputTableUcase:
    .db 0,0,0,0,0,0,0,0,0,0
    .db 'X','T','O','J','E',0,0
    .db ' ','W','S','N','I','D',0,0
    .db 'Z','V','R','M','H','C',0,' '
    .db 'Y','U','Q','L','G','B',0,0,0,0
    .db 'P','K','F','A',0,0,$26,$25,$24,$23,$22,0,0,0
InputTableLcase:
    .db 0,0,0,0,0,0,0,0,0,0
    .db 'x','t','o','j','e',0,0
    .db ' ','w','s','n','i','d',0,0
    .db 'z','v','r','m','h','c',0,' '
    .db 'y','u','q','l','g','b',0,0,0,0
    .db 'p','k','f','a',0,0,$3A,$3F,$21,$2C,$2E,0,0,0
InputTableNum:
    .db 0,0,0,0,0,0,0,0,0,0
    .db '+','-','*','/','^',0,0
    .db $1A,'3','6','9',')',']',0,0
    .db '.','2','5','8','(','[',0,' '
    .db '0','1','4','7',$5C,$7D,0,0,0
    .db $3D,$2C,$D3,$14,$7B,0,0,$27,$5F,$40,$3E,$3C,0,0,0


TitlePic:
;GCP by Andreas Ess, ZCP(by SJ) compressed picture, size = 585 bytes
 .db $00,$3F,$3F,$17,$F0,$8E,$24,$F0,$0F,$F0,$31,$F8,$9E,$24,$F0,$0F
 .db $F0,$31,$F8,$9E,$24,$F0,$09,$F1,$E0,$30,$F8,$9E,$24,$F0,$09,$F1
 .db $E0,$30,$FC,$BE,$24,$F0,$08,$FC,$F0,$31,$FC,$BE,$87,$E0,$8E,$81
 .db $F8,$BB,$83,$9F,$9F,$F0,$2C,$FE,$FE,$8F,$F1,$FF,$83,$FC,$BB,$C3
 .db $9E,$9F,$F8,$2C,$FE,$FE,$9C,$F9,$F3,$C7,$9E,$BB,$C7,$8E,$9E,$B8
 .db $2C,$F7,$EE,$B8,$B9,$E1,$CE,$8E,$B9,$C7,$8E,$9C,$98,$2C,$F7,$EE
 .db $B8,$99,$E1,$CE,$86,$B9,$E6,$8E,$9C,$98,$2C,$F3,$CE,$B8,$9D,$E1
 .db $EE,$87,$B9,$E6,$8E,$98,$98,$2C,$F3,$CE,$B8,$9D,$C1,$EE,$87,$B8
 .db $EE,$8E,$98,$98,$2C,$F0,$8E,$B8,$9D,$C1,$CE,$87,$B8,$FE,$8E,$9C
 .db $98,$2C,$F0,$8E,$B8,$99,$E1,$CE,$86,$B8,$FC,$8E,$9C,$98,$2C,$F0
 .db $8E,$B8,$B9,$E1,$CE,$8E,$B8,$BC,$8E,$9C,$98,$2C,$F0,$8E,$9C,$B9
 .db $E1,$C7,$8E,$B8,$BC,$8E,$9C,$98,$2C,$F0,$8E,$9F,$F1,$E1,$C7,$FC
 .db $B8,$BC,$8F,$9C,$98,$2C,$E0,$86,$8F,$E1,$C1,$C3,$F8,$B0,$B8,$87
 .db $98,$98,$3F,$26,$F0,$3F,$3A,$F0,$3F,$3A,$E0,$3F,$39,$F0,$3F,$39
 .db $F0,$3F,$3A,$E0,$36,$FC,$16,$F0,$19,$49,$0A,$E7,$27,$48,$17,$E0
 .db $19,$F8,$0B,$F3,$28,$F0,$1D,$F0,$14,$F0,$0B,$F0,$8E,$20,$F0,$1E
 .db $F0,$14,$F0,$0B,$F0,$8E,$20,$F0,$15,$E0,$BE,$16,$F0,$0B,$F0,$9F
 .db $12,$E0,$80,$F0,$09,$F0,$BF,$C1,$F8,$9E,$0D,$F0,$80,$FF,$81,$F8
 .db $9F,$87,$F8,$09,$F0,$87,$FC,$9F,$8E,$F8,$9F,$E0,$80,$F0,$81,$FF
 .db $9D,$F0,$BF,$C7,$E0,$09,$F8,$87,$CE,$9E,$8E,$B8,$B9,$E0,$80,$F8
 .db $83,$C7,$9C,$F0,$F3,$E7,$C0,$0B,$FC,$9C,$9C,$F0,$B9,$E3,$C3,$C0
 .db $83,$FF,$9C,$9C,$F3,$C7,$87,$9C,$12,$F8,$F0,$F3,$C1,$E7,$8F,$FE
 .db $0A,$F0,$87,$87,$9C,$F1,$E0,$E7,$16,$F3,$C3,$CE,$87,$9C,$BF,$F0
 .db $08,$F0,$87,$87,$9C,$F1,$E0,$F7,$16,$F3,$C3,$CE,$87,$9C,$B8,$0F
 .db $F0,$87,$87,$9C,$F1,$E0,$F7,$17,$E7,$87,$9C,$8E,$B8,$F0,$0E,$F0
 .db $87,$87,$9C,$F1,$E0,$F7,$17,$E7,$87,$9C,$8E,$B8,$F0,$0E,$F0,$87
 .db $87,$9C,$F1,$E0,$E7,$16,$F3,$C3,$CE,$87,$9C,$B8,$0F,$F0,$87,$87
 .db $9C,$F1,$E1,$E7,$10,$E3,$E7,$8E,$9C,$8E,$B8,$B8,$E0,$80,$F0,$83
 .db $CF,$9C,$F0,$F1,$E7,$10,$48,$8F,$BC,$B8,$9C,$F8,$BF,$C0,$81,$48
 .db $87,$FC,$F3,$E3,$FE,$9C,$0F,$FC,$8F,$F8,$B0,$98,$B8,$9E,$0E,$48
 .db $83,$E0,$E1,$E1,$FC,$98,$19,$F0,$3F,$3A,$F0,$3F,$3A,$F0,$3F,$3A
 .db $F0,$3F,$3A,$F0,$3F,$3A,$E0,$3F,$3F,$31,$C0,$98,$3F,$20,$C0,$0B
 .db $E0,$92,$3F,$34,$C0,$A4,$3F,$0A,$C5,$E7,$DE,$A6,$BC,$81,$80,$C8
 .db $3F,$09,$EE,$B6,$D0,$A9,$B4,$81,$80,$C8,$3F,$0A,$D7,$88,$98,$D2
 .db $C8,$82,$81,$90,$3F,$09,$F4,$A8,$84,$D2,$C8,$82,$81,$90,$3F,$0A
 .db $C7,$90,$F9,$99,$90,$8E,$A1,$C0,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
 .db $3F,$3F,$3F,$3F,$3F,$3F,$35,$80,$00


.end
